<?php
/* $Id: nc_cache_calendar.class.php 3330 2009-06-08 07:09:30Z vadim $ */

/**
 * class nc_cache_calendar
 * @package nc_cache_calendar
 * @category nc_cache
 */
class nc_cache_calendar extends nc_cache {

  /**
   * Constructor method
   *
   * Instantiate in getObject() static method
   * Singleton pattern
   */
  protected function __construct () {
    parent::__construct();
    // set essence
    $this->essence = "calendar";
    // create dir if no exist
    $this->createWritableDir($this->cache_path, $this->essence);
    // append path
    $this->cache_path.= $this->essence."/";
    // bind actions
    $this->core->event->bind( $this, array("updateCatalogue" => "dropCatalogueCache") );
    $this->core->event->bind( $this, array("dropCatalogue" => "dropCatalogueCache") );
    
    $this->core->event->bind( $this, array("updateSubdivision" => "dropSubdivisionCache") );
    $this->core->event->bind( $this, array("dropSubdivision" => "dropSubdivisionCache") );
    
    $this->core->event->bind( $this, array("updateSubClass" => "dropSubClassCache") );
    $this->core->event->bind( $this, array("dropSubClass" => "dropSubClassCache") );
    
    $this->core->event->bind( $this, array("addMessage" => "dropSubClassCache") );
    $this->core->event->bind( $this, array("updateMessage" => "dropSubClassCache") );
    $this->core->event->bind( $this, array("dropMessage" => "dropSubClassCache") );
  }
  
  /**
   * Get or instance self object
   *
   * @return self object
   */
  public static function getObject () {
    // call as static
    static $storage;
    // check inited object
    if ( !isset($storage) ) {
      // init object
      $storage = new self();
    }    
    // return object
    return is_object($storage) ? $storage : false;
  }
  
  /**
   * Add cache on disk method
   *
   * @param int Sub_Class_ID
   * @param int theme ID from `Calendar_Settings`
   * @param string field (Date, Created...)
   * @param string $query_string parameter (date)
   * @param string caching data
   *
   * @return mixed bytes writed or false
   */
  public function add ($cc, $themeID, $field, $query_string, $data) {
    // don't cache in admin mode
    if ($this->admin_mode || $this->inside_admin) return false;
    // if object set in unactive status
    if (!$this->settings["Status_".$this->essence]) return false;
    // check possibility
    if ( !$this->_possibility() ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNRECOGNIZED_OBJECT_CALLING);
    }
    
    // validate
    if (
      !( is_numeric($cc) && $cc > 0 ) ||
      !( is_numeric($themeID) && $themeID > 0 ) ||
      ( $field && !is_string($field) ) ||
      !( $query_string && preg_match("/^([0-9]{1,4}-?){1,3}$/", $query_string) )
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    // unique field string
    $field_str = md5( serialize($field) );
    
    // append audit info
    $this->auditUpdateInfo( array(0, 0, $cc), $query_string, $field_str, "write", true);
    
    // cache info prefix for file
    $cache_prefix = "/* cache created ".time()." */\n";
    
    // check quota and clear old cache or return
    if ( !$this->checkQuota( strlen($cache_prefix.$data) ) ) return false;
    
    // create cc directory
    if ( $this->createWritableDir($this->cache_path, $cc) ) {
      $file_path = $this->cache_path.$cc."/";
    }
    else {
      return false;
    }
    
    // create theme directory
    if ( $this->createWritableDir($file_path, $themeID) ) {
      $file_path.= $themeID."/";
    }
    else {
      return false;
    }
    
    // create field directory
    if ( $this->createWritableDir($file_path, $field_str) ) {
      $file_path.= $field_str."/";
    }
    else {
      return false;
    }
    
    // cache file path
    $file_path.= md5($query_string).".html.php";
    
    // write data into the file
    if ( $bytes_writed = @file_put_contents($file_path, $cache_prefix.$data) ) {
      $this->updateStat($bytes_writed);
      // append audit info
      $this->auditUpdateInfo( array(0, 0, $cc), $query_string, $field_str, "write");
    }
    else {
      throw new Exception ( str_replace("%FILE", $file_path, NETCAT_MODULE_CACHE_CLASS_CANNOT_CREATE_FILE) );
    }
    
    // return result
    return $bytes_writed ? $bytes_writed : false;
  }
  
  /**
   * Read cached file from disk method
   *
   * @param int Sub_Class_ID
   * @param int theme ID from `Calendar_Settings`
   * @param string field (Date, Created...)
   * @param string $query_string parameter (date)
   * @param int cache lifetime
   *
   * @return mixed file data or -1 as false
   */  
  public function read ($cc, $themeID, $field, $query_string, $lifetime = 0) {
    // if object set in unactive status
    if (!$this->settings["Status_".$this->essence]) return -1;
    // check possibility
    if ( !$this->_possibility() ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNRECOGNIZED_OBJECT_CALLING);
    }
    // validate
    if (
      !( is_numeric($cc) && $cc > 0 ) ||
      !( is_numeric($themeID) && $themeID > 0 ) ||
      ( $field && !is_string($field) ) ||
      !( $query_string && preg_match("/^([0-9]{1,4}-?){1,3}$/", $query_string) ) ||
      !is_numeric($lifetime)
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    // unique field string
    $field_str = md5( serialize($field) );
    
    // append audit info
    $this->auditUpdateInfo( array(0, 0, $cc), $query_string, $field_str, "read", true);
    
    // cache file path
    $file_path = $this->cache_path.$cc."/".$themeID."/".$field_str."/".md5($query_string).".html.php";
    
    // check cached file
    if ( file_exists($file_path) ) {
      $content = file_get_contents($file_path);
    }

    // no cache data and cache info in the file
    if (!$content) return -1;

    // check cache expiration
    if ( $this->checkExpire($content, $lifetime) ) {
      // append audit info
      $this->auditUpdateInfo( array(0, 0, $cc), $query_string, $field_str, "read");
      return $content;
    }
    else {
      // delete file and update stat
      if ( is_writable( dirname($file_path) ) ) {
        // update main stat
        $this->updateStat( -filesize($file_path) );
        // delete file
        unlink($file_path);
      }
      return -1;
    }
  }
  
  /**
   * Check parent function by backtrace
   *
   * @return true or false
   */
  private function _possibility () {
    // this function must be called only from "nc_show_calendar" function
    $debug_backtrace = debug_backtrace();
    // get file from calling this method
    $deb_value = $debug_backtrace[2];
    if ( $deb_value['function']=="nc_show_calendar" ) {
      return true;
    }
    return false;
  }
  
  /**
   * Delete cache with setted efficiency (low efficiency == 1)
   *
   * @param int efficiency (1 - low, 2- middle, 3 - good)
   *
   * @return int total deleted bytes
   */
  protected function dropEfficiencyCache ($efficiency) {
    // get cache with efficiency == 1 (low)
    $clear_data = $this->getEfficiencyCache($efficiency);
    // return if empty
    if ( empty($clear_data) ) return false;
    // total deleted bytes
    $total_bytes = 0;
    // try delete cache
    foreach ($clear_data AS $value) {
      $total_bytes+= $this->dropSubClassCache($value['Sub_Class_ID']);
    }
    // return total deleted bytes
    return $total_bytes;
  }

  /**
   * Delete catalogue cache dir from disk method
   *
   * @param mixed Catalogue_ID
   *
   * @return int total deleted bytes
   */   
  public function dropCatalogueCache ($catalogue) {
    // validate
    if (
      !( ( is_numeric($catalogue) && $catalogue > 0 ) || is_array($catalogue) )
    ) {
      return false;
    }
    
    // get all subclasses
    $cc = $this->db->get_col("SELECT DISTINCT `Sub_Class_ID` FROM `Sub_Class`
      WHERE `Catalogue_ID` IN (".join(", ", (array)$catalogue).")");

    // return total deleted bytes
    return !empty($cc) ? $this->dropSubClassCache(0, 0, $cc) : 0;
  }

  /**
   * Delete subdivision cache dir from disk method
   *
   * @param int Catalogue_ID
   * @param mixed Subdivision_ID
   *
   * @return int total deleted bytes
   */  
  public function dropSubdivisionCache ($catalogue, $sub) {
    // validate
    if (
      !( is_numeric($catalogue) && $catalogue > 0 ) ||
      !( (is_numeric($sub) && $sub > 0 ) || is_array($sub) )
    ) {
      return false;
    }
    
    $sub = (array)$sub;
    if ( empty($sub) ) return 0;
    
    // get all subclasses
    $cc = $this->db->get_col("SELECT DISTINCT `Sub_Class_ID` FROM `Sub_Class`
      WHERE `Catalogue_ID` = '".$catalogue."'
      AND `Subdivision_ID` IN (".join(", ", $sub).")");

    // return total deleted bytes
    return !empty($cc) ? $this->dropSubClassCache(0, 0, $cc) : 0;
  }
  
  /**
   * Delete component cache dir from disk method
   *
   * @param dummy
   * @param dummy
   * @param mixed Sub_Class_ID
   *
   * @return int total deleted bytes
   */
  public function dropSubClassCache ($catalogue, $sub, $cc) {
    // validate
    if (
      !( (is_numeric($cc) && $cc > 0 ) || is_array($cc) )
    ) {
      return false;
    }
    // total deleted bytes
    $total_bytes = 0;
    foreach ( (array)$cc AS $cc_id ) {
      // cache file path
      $path = $this->cache_path.$cc_id."/";
      // delete dir
      $total_bytes+= $this->drop($path, true);
    }
    // return total deleted bytes
    return $total_bytes;
  }
  
  /**
   * Delete message cache dir from disk method
   *
   * @param int Sub_Class_ID
   * @param mixed Theme_ID
   *
   * @return int total deleted bytes
   */
  public function dropThemeCache ($cc, $theme) {
    // validate
    if (
      !( (is_numeric($cc) && $cc > 0 ) ) ||
      !( (is_numeric($theme) && $theme > 0) || is_array($theme)  )
    ) {
      return false;
    }
    // total deleted bytes
    $total_bytes = 0;
    foreach ( (array)$theme AS $themeID ) {
      // cache file path
      $path = $this->cache_path.$cc."/".$themeID."/";
      // delete dir
      $total_bytes+= $this->drop($path, true);
    }
    // return total deleted bytes
    return $total_bytes;
  }

  /**
   * Replace nocache marks with nocache blocks in component fields
   * Not used inthis context!
   *
   * dummy function
   */ 
  protected function nocacheReplace () {
    // dummy body
  }

  /**
   * Set nocache marks in component fields
   * Not used inthis context!
   *
   * dummy function
   */   
  public function nocacheStore () {
    // dummy body
  }
  
}

?>