<?php
/* $Id: nc_cache_list.class.php 3330 2009-06-08 07:09:30Z vadim $ */

/**
 * class nc_cache_list
 * @package nc_cache_list
 * @category nc_cache
 */
class nc_cache_list extends nc_cache {

  /**
   * Constructor method
   *
   * Instantiate in getObject() static method
   * Singleton pattern
   */
  protected function __construct () {
    parent::__construct();
    // set essence
    $this->essence = "list";
    // create dir if no exist
    $this->createWritableDir($this->cache_path, $this->essence);
    // append path
    $this->cache_path.= $this->essence."/"; 
    // bind actions
    $this->core->event->bind( $this, array("updateCatalogue" => "dropCatalogueCache") );
    $this->core->event->bind( $this, array("dropCatalogue" => "dropCatalogueCache") );
    
    $this->core->event->bind( $this, array("updateSubdivision" => "dropSubdivisionCache") );
    $this->core->event->bind( $this, array("dropSubdivision" => "dropSubdivisionCache") );
    
    $this->core->event->bind( $this, array("updateSubClass" => "dropSubClassCache") );
    $this->core->event->bind( $this, array("dropSubClass" => "dropSubClassCache") );
    
    $this->core->event->bind( $this, array("updateClass" => "dropClassCache") );
    $this->core->event->bind( $this, array("dropClass" => "dropClassCache") );
    
    $this->core->event->bind( $this, array("addMessage" => "dropSubClassCache") );
    $this->core->event->bind( $this, array("updateMessage" => "dropSubClassCache") );
    $this->core->event->bind( $this, array("dropMessage" => "dropSubClassCache") );
  }
  
  /**
   * Get or instance self object
   *
   * @return self object
   */
  public static function getObject () {
    // call as static
    static $storage;
    // check inited object
    if ( !isset($storage) ) {
      // init object
      $storage = new self();
    }    
    // return object
    return is_object($storage) ? $storage : false;
  }
  
  /**
   * Add cache on disk method
   *
   * @param int Subdivision_ID
   * @param int Sub_Class_ID
   * @param string $query_string parameter
   * @param string caching data
   *
   * @return mixed bytes writed or false
   */
  public function add ($sub, $cc, $query_string, $data) {
    // don't cache in admin mode
    if ($this->admin_mode || $this->inside_admin) return false;
    // if object set in unactive status
    if (!$this->settings["Status_".$this->essence]) return false;
    // check possibility
    if ( !$this->_possibility() ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNRECOGNIZED_OBJECT_CALLING);
    }
    
    // validate
    if (
      !(is_numeric($sub) && $sub > 0) ||
      !(is_numeric($cc) && $cc > 0) ||
      ( $query_string && !is_string($query_string) )
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    // append audit info
    $this->auditUpdateInfo( array(0, $sub, $cc), $query_string, "", "write", true);
    
    // cache info prefix for file
    $cache_prefix = "/* cache created ".time()." */\n";
    
    // check quota and clear old cache or return
    if ( !$this->checkQuota( strlen($cache_prefix.$data) ) ) return false;
    
    // create sub directory
    if ( $this->createWritableDir($this->cache_path, $sub) ) {
      $file_path = $this->cache_path.$sub."/";
    }
    else {
      return false;
    }
    
    // create cc directory
    if ( $this->createWritableDir($file_path, $cc) ) {
      $file_path.= $cc."/";
    }
    else {
      return false;
    }
    
    // cache file path
    $file_path.= md5($query_string);
    
    // set file extension and update data if need it
    if ($new_data = $this->nocacheReplace($data) ) {
      $data = $new_data;
      $file_ext = ".php";
    }
    else {
      $file_ext = ".html.php";
    }
    
    // write data into the file
    if ( $bytes_writed = @file_put_contents($file_path.$file_ext, $cache_prefix.$data) ) {
      $this->updateStat($bytes_writed);
      // append audit info
      $this->auditUpdateInfo( array(0, $sub, $cc), $query_string, "", "write");
    }
    else {
      throw new Exception ( str_replace("%FILE", $file_path.$file_ext, NETCAT_MODULE_CACHE_CLASS_CANNOT_CREATE_FILE) );
    }
    // return result
    return $bytes_writed ? $bytes_writed : false;
  }
  
  /**
   * Read cached file from disk method
   *
   * @param int Subdivision_ID
   * @param int Sub_Class_ID
   * @param string $query_string parameter
   * @param int cache lifetime
   *
   * @return mixed file data or -1 as false
   */  
  public function read ($sub, $cc, $query_string, $lifetime = 0) {
    // if object set in unactive status
    if (!$this->settings["Status_".$this->essence]) return -1;    
    // check possibility
    if ( !$this->_possibility() ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNRECOGNIZED_OBJECT_CALLING);
    }
    // validate
    if (
      !(is_numeric($sub) && $sub > 0) ||
      !(is_numeric($cc) && $cc > 0) ||
      ( $query_string && !is_string($query_string) ) ||
      !is_numeric($lifetime)
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    // append audit info
    $this->auditUpdateInfo( array(0, $sub, $cc), $query_string, "", "read", true);
    // startup values
    $result = false;
    // cache file path
    $file_path = $this->cache_path.$sub."/".$cc."/".md5($query_string);
    // check cached file
    switch (true) {
      case file_exists($file_path.".html.php"):
        $file_exist_path = $file_path.".html.php";
        $content = file_get_contents($file_exist_path);
        $content_eval = false;
      break;
      case file_exists($file_path.".php"):
        $file_exist_path = $file_path.".php";
        $content = file_get_contents($file_exist_path);
        $content_eval = true;
      break;
    }
    // no cache data and cache info in the file
    if (!$content) return -1;
    // check cache expiration
    if ( $this->checkExpire($content, $lifetime) ) {
      $result = array($content, $content_eval);
      // append audit info
      $this->auditUpdateInfo( array(0, $sub, $cc), $query_string, "", "read");
    }
    else {
      // delete file and update stat
      if ( is_writable( dirname($file_exist_path) ) ) {
        // update main stat
        $this->updateStat( -filesize($file_exist_path) );
        // delete file
        unlink($file_exist_path);
      }
    }
    // return false
    return $result ? $result : -1;
  }
  
  /**
   * Check s_list_class() parent function by backtrace
   *
   * @return true or false
   */
  private function _possibility () {
    // this function must be called only from s_list_class() function
    $debug_backtrace = debug_backtrace();
    // get function from calling this method
    $deb_value = $debug_backtrace[2];
    if ($deb_value['function'] == "s_list_class" || $deb_value['function'] == "nc_objects_list") {
      return true;
    }
    return false;
  }

  /**
   * Delete cache with setted efficiency (low efficiency == 1)
   *
   * @param int efficiency (1 - low, 2- middle, 3 - good)
   *
   * @return int total deleted bytes
   */
  protected function dropEfficiencyCache ($efficiency) {
    // get cache with efficiency == 1 (low)
    $clear_data = $this->getEfficiencyCache($efficiency);
    // return if empty
    if ( empty($clear_data) ) return false;
    // total deleted bytes
    $total_bytes = 0;
    // try delete cache
    foreach ($clear_data AS $value) {
      $total_bytes+= $this->dropSubClassCache($value['Subdivision_ID'], $value['Sub_Class_ID']);
    }
    // return total deleted bytes
    return $total_bytes;
  }
  
  /**
   * Delete catalogue cache dir from disk method
   *
   * @param mixed Catalogue_ID
   *
   * @return int total deleted bytes
   */  
  public function dropCatalogueCache ($catalogue) {
    // validate
    if (
      !( ( is_numeric($catalogue) && $catalogue > 0 ) || is_array($catalogue) )
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    
    $catalogue = (array)$catalogue;
    if ( empty($catalogue) ) return 0;
    
    // get all catalogues subdivisions
    $sub = $this->db->get_col("SELECT DISTINCT `Subdivision_ID` FROM `Subdivision`
      WHERE `Catalogue_ID` IN (".join(", ", $catalogue).")");

    // return total deleted bytes
    return !empty($sub) ? $this->dropSubdivisionCache(0, $sub) : 0;
  }
  
  /**
   * Delete subdivision cache dir from disk method
   *
   * @param dummy
   * @param int Subdivision_ID
   *
   * @return int total deleted bytes
   */  
  public function dropSubdivisionCache ($catalogue, $sub) {
    // validate
    if (
      !( ( is_numeric($sub) && $sub > 0 ) || is_array($sub) )
    ) {
      throw new Exception (NETCAT_MODULE_CACHE_CLASS_UNCORRECT_DATA_FORMAT);
    }
    // total deleted bytes
    $total_bytes = 0;
    foreach ( (array)$sub AS $sub_id ) {
      // cache file path
      $path = $this->cache_path.$sub_id."/";
      // delete dir
      $total_bytes+= $this->drop($path, true);
    }
    // return total deleted bytes
    return $total_bytes;
  }
  
  /**
   * Delete cc cache dir from disk method
   *
   * @param dummy
   * @param int Subdivision_ID
   * @param mixed Sub_Class_ID
   *
   * @return int total deleted bytes
   */
  public function dropSubClassCache ($catalogue, $sub, $cc) {
    // validate
    if (
      !(is_numeric($sub) && $sub > 0 ) ||
      !( (is_numeric($cc) && $cc > 0 ) || is_array($cc) )
    ) {
      return false;
    }
    // total deleted bytes
    $total_bytes = 0;
    foreach ( (array)$cc AS $cc_id ) {
      // cache file path
      $path = $this->cache_path.$sub."/".$cc_id."/";
      // delete dir
      $total_bytes+= $this->drop($path, true);
    }
    // return total deleted bytes
    return $total_bytes;
  }

  /**
   * Delete component cache dir from disk method
   *
   * @param int Class_ID
   *
   * @return int total deleted bytes
   */
  public function dropClassCache ($class) {
    // validate
    if (
      !( (is_numeric($class) && $class > 0 ) || is_array($class) )
    ) {
      return false;
    }
    
    $class = (array)$class;
    if ( empty($class) ) return 0;
    
    // get all catalogues subdivisions
    $data = $this->db->get_results("SELECT DISTINCT `Subdivision_ID`, `Sub_Class_ID` FROM `Sub_Class`
      WHERE `Class_ID` IN (".join(", ", $class).")", ARRAY_A);
      
    $total_deleted = 0;
    if ( !empty($data) ) {
      foreach ($data as $value) {
        $total_deleted += $this->dropSubClassCache(0, $value['Subdivision_ID'], $value['Sub_Class_ID']);
      }      
    }

    // return total deleted bytes
    return $total_deleted;
  }
  
  /**
   * Set nocache marks in component fields
   *
   * @param array component data
   *
   * @param int counted nocache block
   */   
  public function nocacheStore (&$data) {
    // parsed values with need sequence
    $this->nocacheStoreKeys = array("FormPrefix", "RecordTemplate", "FormSuffix");
    // call parent
    return parent::nocacheStore($data);
  }
  
}

?>