<?php
/* $Id: nc_subdivision.class.php 3338 2009-06-08 12:02:47Z vadim $ */

class nc_Subdivision extends nc_Essence {
  
  protected $db;
  
  /**
   * Constructor function
   */  
  public function __construct () {
    // load parent constructor
    parent::__construct();
    
    // system superior object
    $nc_core = nc_Core::get_object();
    // system db object
    if ( is_object($nc_core->db) ) $this->db = $nc_core->db;
    
  }
  
  /**
   * Get subdivision data by the URI parameter
   *
   * @param URI string
   * @param bool use returned value as current catalogue data
   * @param bool reset stored data in the static variable
   *
   * @return subdivision data associative array
   */
  public function set_current_by_uri ($reset = false) {
    // system superior object
    $nc_core = nc_Core::get_object();
    
    // call as static
    static $storage = array();
    
    // URI
    $uri = $nc_core->url->get_parsed_url('path');
    
    // validate Hidden_URL
    if ( !$this->validate_hidden_url($uri) ) return false;
    
    // check inited object
    if ( empty($storage) || $reset ) {
      
      // find date in url
      preg_match("|
        / (\d{4}) /
        (?:
          (\d{2}) /
        )?
        (?:
          (\d{2}) /
        )?
      |x", $uri, $regs);
      // date found
      if ($regs) {
        // crop date symbols
        $uri = str_replace($regs[0], "/", $uri);
      }
    
      $storage = $this->db->get_row("SELECT * FROM `Subdivision` WHERE `Catalogue_ID` = '".intval( $nc_core->catalogue->get_current("Catalogue_ID") )."' AND `Hidden_URL` = '".$this->db->escape($uri)."'", ARRAY_A);

      // processing system fields
      $this->current = $this->convert_system_vars($storage);
      
      // inherit current data, only after then setted $this->current!
      $this->current = $this->inherit($this->current);      
    }
    
    return $storage;
  }
  
  public function validate_hidden_url ($url) {
    // validate Hidden_URL
    return preg_match("/^[\/_a-z0-9-]+$/i", $url);  
  }
  
  protected function inherit ($sub_env) {
    // system superior object
    $nc_core = nc_Core::get_object();
    
    $sub = $sub_env["Subdivision_ID"];
    
    $this->_level_count[$sub] = 0;
    $this->_parent_tree[$sub][] = "";
    // local variables
    $level_count = $this->_level_count[$sub];
    $parent_tree = $this->_parent_tree[$sub];

    $parent_tree[$level_count] = $sub_env;
    $level_count++;

    $parent_sub = $sub_env["Parent_Sub_ID"];
    
    while ($parent_sub) {
      $parent_sub_env = $this->get_by_id($parent_sub);

      if (!$sub_env["Template_ID"]) {
        $sub_env["Template_ID"] = $parent_sub_env["Template_ID"];
        $sub_env["TemplateSettings"] = $parent_sub_env["TemplateSettings"];
      }
      if (!$sub_env["Read_Access_ID"]) $sub_env["Read_Access_ID"] = $parent_sub_env["Read_Access_ID"];
      if (!$sub_env["Write_Access_ID"]) $sub_env["Write_Access_ID"] = $parent_sub_env["Write_Access_ID"];
      if (!$sub_env["Edit_Access_ID"]) $sub_env["Edit_Access_ID"] = $parent_sub_env["Edit_Access_ID"];
      if (!$sub_env["Subscribe_Access_ID"]) $sub_env["Subscribe_Access_ID"] = $parent_sub_env["Subscribe_Access_ID"];
      if ( $nc_core->modules->get_by_keyword("cache") ) {
        if (!$sub_env["Cache_Access_ID"]) $sub_env["Cache_Access_ID"] = $parent_sub_env["Cache_Access_ID"];
        if (!$sub_env["Cache_Lifetime"]) $sub_env["Cache_Lifetime"] = $parent_sub_env["Cache_Lifetime"];
      }
      if (!$sub_env["Moderation_ID"]) $sub_env["Moderation_ID"] = $parent_sub_env["Moderation_ID"];

      $parent_tree[$level_count] = $parent_sub_env;
      $sub_env = $this->inherit_system_fields($this->essence, $parent_tree[$level_count], $sub_env);

      $level_count++;

      $parent_sub = $parent_sub_env["Parent_Sub_ID"];
    }

    $catalogue_env = $nc_core->catalogue->get_by_id($sub_env["Catalogue_ID"]);

    $sub_env = $this->inherit_system_fields("Catalogue", $catalogue_env, $sub_env);

    if (!$sub_env["Template_ID"]) {
     $sub_env["Template_ID"] = $catalogue_env["Template_ID"];
     $sub_env["TemplateSettings"] = $catalogue_env["TemplateSettings"];
    }

    if (!$sub_env["Read_Access_ID"]) $sub_env["Read_Access_ID"] = $catalogue_env["Read_Access_ID"];
    if (!$sub_env["Write_Access_ID"]) $sub_env["Write_Access_ID"] = $catalogue_env["Write_Access_ID"];
    if (!$sub_env["Edit_Access_ID"]) $sub_env["Edit_Access_ID"] = $catalogue_env["Edit_Access_ID"];
    if (!$sub_env["Subscribe_Access_ID"]) $sub_env["Subscribe_Access_ID"] = $catalogue_env["Subscribe_Access_ID"];
    if ( $nc_core->modules->get_by_keyword("cache") ) {
      if (!$sub_env["Cache_Access_ID"]) $sub_env["Cache_Access_ID"] = $catalogue_env["Cache_Access_ID"];
      if (!$sub_env["Cache_Lifetime"]) $sub_env["Cache_Lifetime"] = $catalogue_env["Cache_Lifetime"];
    }
    if (!$sub_env["Moderation_ID"]) $sub_env["Moderation_ID"] = $catalogue_env["Moderation_ID"];

    $parent_tree[$level_count] = $catalogue_env;
    
    // store data
    $this->_level_count[$sub] = $level_count;
    $this->_parent_tree[$sub] = $parent_tree;
    
    return $sub_env;
  }
  
  public function get_parent_tree ($sub) {
    // get data
    $this->get_by_id($sub);
    // check
    if ( is_array($this->_parent_tree[$sub]) && !empty($this->_parent_tree[$sub]) ) {
      return $this->_parent_tree[$sub];
    }
    // parent sub array not found
    return false;
  }
  
  public function get_level_count ($sub) {
    // get data
    $this->get_by_id($sub);
    // check
    if ( isset($this->_level_count[$sub]) ) {
      return $this->_level_count[$sub];
    }
    // level count array not found
    return false;
  }
  
}


?>