<?php
/* $Id: nc_event.class.php 3249 2009-05-13 13:03:50Z vadim $ */

class nc_Event extends nc_System {
  
  private $_binded_obj, $_events_arr;
  
  public function __construct () {
    // load parent constructor
    parent::__construct();
    
    // collect objects for events
    $this->_binded_obj = array();
    
    // allowed events
    $this->_events_arr = array(
      // add actions
      "addCatalogue",
      "addSubdivision",
      "addSubClass",
      "addClass",
      "addMessage",
      "addSystemTable", # unused
      "addTemplate",
      "addUser",
      // update actions
      "updateCatalogue",
      "updateSubdivision",
      "updateSubClass",
      "updateClass",
      "updateMessage",
      "updateSystemTable",
      "updateTemplate",
      "updateUser",
      // drop actions
      "dropCatalogue",
      "dropSubdivision",
      "dropSubClass",
      "dropClass",
      "dropMessage",
      "dropSystemTable", # unused
      "dropTemplate",
      "dropUser"
    );
  }

  /**
   * Add object to the listen mode
   *
   * @param object examine object
   */
  public function bind (&$object, $events_remap_arr = false) {
    // add object in array
    if ( !in_array($object, $this->_binded_obj) ) {
      // set empty remap array
      if (!$events_remap_arr) $events_remap_arr = array();
      // bind object with remap array
      $this->_binded_obj[] = array('object' => $object, 'remap' => $events_remap_arr);
      //echo get_class($object)." - ".print_r($events_remap_arr, 1)."<br/>";
    }
    return;
  }
  
  /**
   * Event processor
   * call objects function for current event
   *
   */
  public function execute () {
    // get function args
    $args = func_get_args();
    // check args
    if ( empty($args) || empty($this->_binded_obj) ) return false;
    // event
    $event = array_shift($args);
    // check base system event
    if ( !( $event && in_array($event, $this->_events_arr) ) ) {
      return false;
    }
    
    //echo "<h2>$event</h2>";
    foreach ($this->_binded_obj as $object) {
      // check remaping
      if ( !empty($object['remap']) ) {
        // remap event method
        $remap_event = $object['remap'][$event] ? $object['remap'][$event] : "";
      }
      // determine event for this iteration
      $_event = $remap_event ? $remap_event : $event;
      // check and execute observer method
      if ( is_callable(  array($object['object'], $_event) ) ) {
        call_user_func_array( array($object['object'], $_event), $args );
        //echo "call_user_func_array( array(".$object['object']."<b>[".get_class($object['object'])."]</b>, ".$_event."), ".print_r($args, 1)." );<br/>";
      }
    }
    
    return;
  }
  
}

?>