<?php
/* $Id: nc_input.class.php 3276 2009-05-26 14:15:54Z vadim $ */

class nc_Input extends nc_System {
  
  private $_variables;
  
  public function __construct () {
    // load parent constructor
    parent::__construct();
    
    $this->_variables = array(
      // superglobals inside superglobals are not allowed
      "_SERVER", "_ENV", "_FILES", "_GET", "_POST", "_COOKIE", "GLOBALS",
       // netcat configuration variables
      "ADMIN_AUTHTIME", "ADMIN_AUTHTYPE", "ADMIN_LANGUAGE",
      "AUTHORIZATION_TYPE", "AUTHORIZE_BY", "AUTH_USER_GROUP", "AUTH_USER_ID",
      "DIRCHMOD", "FILECHMOD", "CHARSET", "SOURCE_CHARSET",
      "DOCUMENT_ROOT", "DOC_DOMAIN", "DOMAIN_NAME", "EDIT_DOMAIN", "HTTP_HOST",
      "HTTP_IMAGES_PATH", "HTTP_FILES_PATH", "HTTP_DUMP_PATH", "HTTP_ROOT_PATH", "HTTP_CACHE_PATH",
        "ADMIN_PATH", "ADMIN_TEMPLATE",
      "MYSQL_CHARSET", "MYSQL_DB_NAME", "MYSQL_HOST", "MYSQL_PASSWORD", "MYSQL_USER", "SHOW_MYSQL_ERRORS",
      "SYSTEM_FOLDER", "SUB_FOLDER", "NETCAT_ROOT_FOLDER", "ROOT_FOLDER", "TMP_FOLDER", "FILES_FOLDER",
        "MODULE_FOLDER", "INCLUDE_FOLDER", "ADMIN_TEMPLATE_FOLDER", "DUMP_FOLDER", "CACHE_FOLDER", "ADMIN_FOLDER",
      "PHP_TYPE", "REDIRECT_STATUS", "BBCODE", "AJAX_SAVER", "DEVELOPER_NAME", "DEVELOPER_URL", "use_gzip_compression"
    );
    
  }
  
  public function clear_system_vars ($result) {
    // clean up array
    foreach ($this->_variables as $var) {
      // unset system variables if they setted in superglobals arrays
      unset ($result[$var]);
    }
    // return array
    return $result;
  }
  
  public function prepare_extract () {
    // system superior object
    $nc_core = nc_Core::get_object();
    
    // superglobal arrays
    $superglobals = array("_GET" => $_GET, "_POST" => $_POST, "_COOKIE" => $_COOKIE, "_FILES" => $_FILES, "_ENV" => $_ENV, "_SERVER" => $_SERVER);
    // set default
    $result = array();
    
    // merge superglobals arrays
    foreach ($superglobals as $key => $super_array) {
      // build result array
      $result = array_merge($result, $super_array);
      // set internal data from superglobal arrays
      $this->$key = $super_array;
    }
    
    // clean up input
    $result = $this->clear_system_vars($result);
    
    // set internal system object variables
    foreach ($this->_variables as $var) {
      // skip superglobals
      if ( array_key_exists($var, $superglobals) ) continue;
      // set internal system object variables. They getted from vars.inc.php
      global $$var;
      $nc_core->set_variable($var, $$var);
    }

    if ($_POST["NC_HTTP_REQUEST"]) {
      $result = $nc_core->utf8->array_utf2win($result);
    }
    
    if ( !get_magic_quotes_gpc() ) {
      $result = $this->recursive_add_slashes($result);
    }
    
    return $result;
  }
  
  public function recursive_add_slashes ($input) {
    if ( !is_array($input) ) {
      return addslashes($input);
    }
    $output = array();

    foreach ($input as $k => $v) {
       $output[$k] = is_array($v) ? $this->recursive_add_slashes($v) : addslashes($v);
    }

    return $output;
  }
  
  public function fetch_get ($item = "") {
    
    if ( empty($this->_GET) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_GET) ? $this->_GET[$item] : "";
    }
    else {
      return $this->_GET;
    }
    
  }
  
  public function fetch_post ($item = "") {
    
    if ( empty($this->_POST) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_POST) ? $this->_POST[$item] : "";
    }
    else {
      return $this->_POST;
    }
    
  }
  
  public function fetch_cookie ($item = "") {
    
    if ( empty($this->_COOKIE) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_COOKIE) ? $this->_COOKIE[$item] : "";
    }
    else {
      return $this->_COOKIE;
    }
    
  }
  
  public function fetch_session ($item = "") {
    
    if ( empty($this->_SESSION) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_SESSION) ? $this->_SESSION[$item] : "";
    }
    else {
      return $this->_SESSION;
    }
    
  }
  
  public function fetch_files ($item = "") {
    
    if ( empty($this->_FILES) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_FILES) ? $this->_FILES[$item] : "";
    }
    else {
      return $this->_FILES;
    }
    
  }
  
  public function fetch_get_post ($item = "") {
    
    if ( empty($this->_GET) && empty($this->_POST) ) return false;
    
    if ( $item ) {
      return array_key_exists($item, $this->_GET) ? $this->_GET[$item] : (array_key_exists($item, $this->_POST) ? $this->_POST[$item] : "");
    }
    else {
      return array_merge($this->_GET, $this->_POST);
    }
    
  }
  
}

?>